#include "../../include/rcore.h"
#include "../loss.h"

/* compute the log-likelihood of data, either individually or as a set. */
SEXP data_loglikelihood(SEXP fitted, SEXP data, SEXP by_sample, SEXP keep,
    SEXP debug) {

bool by = isTRUE(by_sample);
int ndata = length(VECTOR_ELT(data, 0));
double *res_sample = NULL, loglik = 0;
SEXP result_sample = R_NilValue;

  /* allocate the vector of the individual log-likelihoods, if needed. */
  if (by) {

    PROTECT(result_sample = allocVector(REALSXP, ndata));
    res_sample = REAL(result_sample);
    memset(res_sample, '\0', ndata * sizeof(double));

  }/*THEN*/

  loglik = c_entropy_loss(fitted, data, ndata, by, res_sample, NULL, keep,
             TRUE, TRUE, isTRUE(debug));

  if (by) {

    UNPROTECT(1);

    return result_sample;

  }/*THEN*/
  else {

    /* the entropy is the average log-likelihood, so scale it back. */
    loglik *= -ndata;

    return ScalarReal(loglik);

  }/*ELSE*/

}/*DATA_LOGLIKELIHOOD*/

